var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import webpack from 'webpack';
import * as path from 'path';
const certificateManager = require('@rushstack/debug-certificate-manager');
const certificateStore = new certificateManager.CertificateStore();
import { AsyncComponentPlugin } from '@microsoft/spfx-heft-plugins/lib/plugins/webpackConfigurationPlugin/webpackPlugins/AsyncComponentPlugin';
import ForkTsCheckerWebpackPlugin from 'fork-ts-checker-webpack-plugin';
import { ClearCssModuleDefinitionsPlugin } from '../plugins/ClearCssModuleDefinitionsPlugin';
import { TypeScriptResourcesPlugin } from '../plugins/TypeScriptResourcesPlugin';
import { freePortIfInUse, getExternalComponents, getJSONFile } from './helpers';
const packageJson = getJSONFile('package.json');
const hasESLint = !!packageJson.devDependencies['@typescript-eslint/parser'];
const rootFolder = path.resolve(process.cwd());
export function createBaseConfig(cli) {
    return __awaiter(this, void 0, void 0, function* () {
        let port = 0;
        if (!cli.isLibraryComponent) {
            port = 4321;
        }
        else {
            if (!cli.port) {
                port = 4320;
            }
            else {
                port = cli.port;
            }
        }
        yield freePortIfInUse(port);
        const host = 'https://localhost:' + port;
        const cssLoader = require.resolve('css-loader');
        const themedLoader = require.resolve('@microsoft/loader-load-themed-styles');
        const externalComponents = getExternalComponents();
        const baseConfig = {
            target: 'web',
            mode: 'development',
            devtool: 'source-map',
            resolve: {
                extensions: ['.ts', '.tsx', '.js'],
                modules: ['node_modules']
            },
            context: rootFolder,
            module: {
                rules: [
                    {
                        use: [
                            {
                                loader: require.resolve('ts-loader'),
                                options: {
                                    transpileOnly: true,
                                    compilerOptions: {
                                        declarationMap: false
                                    }
                                },
                            }
                        ],
                        test: /\.tsx?$/,
                        exclude: /node_modules/
                    },
                    {
                        use: [{
                                loader: require.resolve('file-loader'),
                                options: {
                                    name: '[name]_[hash].[ext]',
                                    esModule: false
                                }
                            }],
                        test: /\.(jpe?g|png|woff|eot|ttf|svg|gif|dds)$/i
                    },
                    {
                        use: [{
                                loader: 'html-loader'
                            }],
                        test: /\.html$/
                    },
                    {
                        test: /\.css$/,
                        use: [
                            {
                                loader: themedLoader,
                                options: {
                                    async: true
                                }
                            },
                            {
                                loader: cssLoader,
                                options: {
                                    esModule: false,
                                    modules: false
                                }
                            }
                        ]
                    },
                    {
                        test: function (fileName) {
                            return fileName.endsWith('.module.scss');
                        },
                        use: [
                            {
                                loader: themedLoader,
                                options: {
                                    async: true
                                }
                            },
                            require.resolve('spfx-css-modules-typescript-loader'),
                            {
                                loader: cssLoader,
                                options: {
                                    esModule: false,
                                    modules: {
                                        localIdentName: '[local]_[hash:base64:8]',
                                        mode: (resourcePath) => {
                                            if (resourcePath.endsWith('.css') && /node_modules/gi.test(resourcePath)) {
                                                return 'global';
                                            }
                                            return 'local';
                                        }
                                    }
                                }
                            },
                            {
                                loader: require.resolve('sass-loader'),
                                options: {
                                    implementation: require('node-sass')
                                }
                            }
                        ]
                    },
                    {
                        test: function (fileName) {
                            return !fileName.endsWith('.module.scss') && fileName.endsWith('.scss');
                        },
                        use: [
                            {
                                loader: themedLoader,
                                options: {
                                    async: true
                                }
                            },
                            {
                                loader: cssLoader,
                                options: {
                                    esModule: false,
                                    modules: false
                                }
                            },
                            {
                                loader: require.resolve('sass-loader'),
                                options: {
                                    implementation: require('node-sass')
                                }
                            }
                        ]
                    }
                ]
            },
            plugins: [
                new AsyncComponentPlugin({
                    externalComponents
                }),
                new webpack.WatchIgnorePlugin([path.resolve(rootFolder, 'temp')]),
                new ForkTsCheckerWebpackPlugin({
                    eslint: hasESLint ? {
                        files: './src/**/*.{ts,tsx}',
                        enabled: true
                    } : undefined,
                    async: true
                }),
                new TypeScriptResourcesPlugin(),
                new ClearCssModuleDefinitionsPlugin({
                    rootFolder
                }),
                new webpack.DefinePlugin({
                    'process.env.NODE_ENV': JSON.stringify(process.env.NODE_ENV),
                    'DEBUG': JSON.stringify(true)
                })
            ],
            devServer: {
                hot: false,
                contentBase: rootFolder,
                publicPath: host + '/dist/',
                host: 'localhost',
                port: port,
                disableHostCheck: true,
                historyApiFallback: true,
                writeToDisk: cli.isLibraryComponent,
                headers: {
                    'Access-Control-Allow-Origin': '*',
                },
                https: {
                    cert: certificateStore.certificateData,
                    key: certificateStore.keyData
                }
            },
        };
        return baseConfig;
    });
}
//# sourceMappingURL=baseConfig.js.map