var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import * as path from 'path';
import CopyPlugin from 'copy-webpack-plugin';
import del from 'del';
import { merge } from 'webpack-merge';
import { DynamicLibraryPlugin } from '../plugins/DynamicLibraryPlugin';
import { addCopyLocalExternals, addCopyLocalizedResources, checkVersions, createLocalExternals, getEntryPoints, getJSONFile } from './helpers';
import { createBaseConfig } from './baseConfig';
import { applyServeSettings } from '../settings';
const rootFolder = path.resolve(process.cwd());
const { transformConfig, webpackConfig } = require(path.join(rootFolder, 'fast-serve/webpack.extend'));
const createConfig = function () {
    return __awaiter(this, void 0, void 0, function* () {
        checkVersions();
        const settings = getJSONFile('fast-serve/config.json');
        const baseConfig = yield createBaseConfig(settings.cli);
        del.sync(['dist/*.js', 'dist/*.map'], { cwd: rootFolder });
        const originalWebpackConfig = getJSONFile('temp/_webpack_config.json');
        baseConfig.externals = originalWebpackConfig.externals;
        baseConfig.output = originalWebpackConfig.output;
        baseConfig.entry = getEntryPoints(originalWebpackConfig.entry);
        baseConfig.output.publicPath = `https://${baseConfig.devServer.host}:${baseConfig.devServer.port}/dist/`;
        const manifest = getJSONFile('temp/manifests.json');
        const { localizedResources, externals } = getJSONFile('config/config.json');
        const modulesMap = {};
        const originalEntries = Object.keys(originalWebpackConfig.entry);
        for (const jsModule of manifest) {
            if (jsModule.loaderConfig
                && jsModule.loaderConfig.entryModuleId
                && originalEntries.indexOf(jsModule.loaderConfig.entryModuleId) !== -1) {
                const entryModuleId = jsModule.loaderConfig.entryModuleId;
                modulesMap[entryModuleId + '.js'] = {
                    id: jsModule.id,
                    version: jsModule.version,
                    path: jsModule.loaderConfig.scriptResources[entryModuleId].path
                };
            }
        }
        baseConfig.output.filename = function (pathInfo) {
            const entryPointName = pathInfo.chunk.name + '.js';
            if (!modulesMap[entryPointName]) {
                return '[name].js';
            }
            return modulesMap[entryPointName].path;
        };
        baseConfig.plugins.push(new DynamicLibraryPlugin({
            modulesMap: modulesMap,
            libraryName: originalWebpackConfig.output.library
        }));
        const patterns = addCopyLocalizedResources(localizedResources);
        baseConfig.plugins.push(new CopyPlugin({
            patterns
        }));
        applyServeSettings(baseConfig);
        const localExternals = createLocalExternals(externals);
        const localExternalsPatterns = addCopyLocalExternals(localExternals, manifest, originalEntries);
        if (localExternalsPatterns && localExternalsPatterns.length > 0) {
            baseConfig.plugins.push(new CopyPlugin({
                patterns: localExternalsPatterns
            }));
        }
        return baseConfig;
    });
};
export const resultConfig = () => __awaiter(void 0, void 0, void 0, function* () { return merge(transformConfig(yield createConfig()), webpackConfig); });
//# sourceMappingURL=configureWebPack.js.map